@@ -9,11 +9,11 @@
 import json
 import logging
 import os
+import re
 import shutil
 import subprocess
 from functools import lru_cache  # pylint: disable=syntax-error
 
-
 # exiftool -stay_open commands outputs this EOF marker after command is run
 EXIFTOOL_STAYOPEN_EOF = "{ready}"
 EXIFTOOL_STAYOPEN_EOF_LEN = len(EXIFTOOL_STAYOPEN_EOF)
@@ -33,8 +33,8 @@ def get_exiftool_path():
 
 
 class _ExifToolProc:
-    """ Runs exiftool in a subprocess via Popen
-        Creates a singleton object """
+    """Runs exiftool in a subprocess via Popen
+    Creates a singleton object"""
 
     def __new__(cls, *args, **kwargs):
         """ create new object or return instance of already created singleton """
@@ -44,20 +44,20 @@ class _ExifToolProc:
         return cls.instance
 
     def __init__(self, exiftool=None):
-        """ construct _ExifToolProc singleton object or return instance of already created object
-            exiftool: optional path to exiftool binary (if not provided, will search path to find it) """
+        """construct _ExifToolProc singleton object or return instance of already created object
+        exiftool: optional path to exiftool binary (if not provided, will search path to find it)"""
 
         if hasattr(self, "_process_running") and self._process_running:
             # already running
-            if exiftool is not None:
+            if exiftool is not None and exiftool != self._exiftool:
                 logging.warning(
                     f"exiftool subprocess already running, "
                     f"ignoring exiftool={exiftool}"
                 )
             return
 
-        self._exiftool = exiftool or get_exiftool_path()
         self._process_running = False
+        self._exiftool = exiftool or get_exiftool_path()
         self._start_proc()
 
     @property
@@ -106,8 +106,8 @@ class _ExifToolProc:
 
     def _stop_proc(self):
         """ stop the exiftool process if it's running, otherwise, do nothing """
+
         if not self._process_running:
-            logging.warning("exiftool process is not running")
             return
 
         self._process.stdin.write(b"-stay_open\n")
@@ -132,19 +132,23 @@ class _ExifToolProc:
 class ExifTool:
     """ Basic exiftool interface for reading and writing EXIF tags """
 
-    def __init__(self, filepath, exiftool=None, overwrite=True):
-        """ Create ExifTool object
+    def __init__(self, filepath, exiftool=None, overwrite=True, flags=None):
+        """Create ExifTool object
 
         Args:
-                file: path to image file
-                exiftool: path to exiftool, if not specified will look in path
-                overwrite: if True, will overwrite image file without creating backup, default=False
+            file: path to image file
+            exiftool: path to exiftool, if not specified will look in path
+            overwrite: if True, will overwrite image file without creating backup, default=False
+            flags: optional list of exiftool flags to prepend to exiftool command when writing metadata (e.g. -m or -F)
+
         Returns:
             ExifTool instance
         """
         self.file = filepath
         self.overwrite = overwrite
+        self.flags = flags or []
         self.data = {}
+        self.warning = None
         self.error = None
         # if running as a context manager, self._context_mgr will be True
         self._context_mgr = False
@@ -153,16 +157,17 @@ class ExifTool:
         self._read_exif()
 
     def setvalue(self, tag, value):
-        """ Set tag to value(s); if value is None, will delete tag
+        """Set tag to value(s); if value is None, will delete tag
 
         Args:
             tag: str; name of tag to set
             value: str; value to set tag to
-        
+
         Returns:
             True if success otherwise False
-            
+
             If error generated by exiftool, returns False and sets self.error to error string
+            If warning generated by exiftool, returns True (unless there was also an error) and sets self.warning to warning string
             If called in context manager, returns True (execution is delayed until exiting context manager)
         """
 
@@ -175,29 +180,30 @@ class ExifTool:
             self._commands.extend(command)
             return True
         else:
-            _, self.error = self.run_commands(*command)
-            return self.error is None
+            _, _, error = self.run_commands(*command)
+            return error is None
 
     def addvalues(self, tag, *values):
-        """ Add one or more value(s) to tag
+        """Add one or more value(s) to tag
             If more than one value is passed, each value will be added to the tag
 
         Args:
             tag: str; tag to set
             *values: str; one or more values to set
-        
+
         Returns:
             True if success otherwise False
-            
+
             If error generated by exiftool, returns False and sets self.error to error string
+            If warning generated by exiftool, returns True (unless there was also an error) and sets self.warning to warning string
             If called in context manager, returns True (execution is delayed until exiting context manager)
-            
+
         Notes: exiftool may add duplicate values for some tags so the caller must ensure
                the values being added are not already in the EXIF data
-               For some tags, such as IPTC:Keywords, this will add a new value to the list of keywords, 
+               For some tags, such as IPTC:Keywords, this will add a new value to the list of keywords,
                but for others, such as EXIF:ISO, this will literally add a value to the existing value.
                It's up to the caller to know what exiftool will do for each tag
-               If setvalue called before addvalues, exiftool does not appear to add duplicates, 
+               If setvalue called before addvalues, exiftool does not appear to add duplicates,
                but if addvalues called without first calling setvalue, exiftool will add duplicate values
         """
         if not values:
@@ -216,11 +222,11 @@ class ExifTool:
             self._commands.extend(command)
             return True
         else:
-            _, self.error = self.run_commands(*command)
-            return self.error is None
+            _, _, error = self.run_commands(*command)
+            return error is None
 
     def run_commands(self, *commands, no_file=False):
-        """ Run commands in the exiftool process and return result.
+        """Run commands in the exiftool process and return result.
 
         Args:
                 *commands: exiftool commands to run
@@ -228,11 +234,12 @@ class ExifTool:
                         by default, all commands will be run against self.file
                         use no_file=True to run a command without passing the filename
         Returns:
-            (output, errror)
+            (output, warning, errror)
             output: bytes is containing output of exiftool commands
-            error: if exiftool generated an error, bytes containing error string otherwise None
+            warning: if exiftool generated warnings, string containing warning otherwise empty string
+            error: if exiftool generated errors, string containing otherwise empty string
 
-        Note: Also sets self.error if error generated.
+        Note: Also sets self.warning and self.error if warning or error generated.
         """
         if not (hasattr(self, "_process") and self._process):
             raise ValueError("exiftool process is not running")
@@ -245,7 +252,14 @@ class ExifTool:
             commands.append("-overwrite_original")
 
         filename = os.fsencode(self.file) if not no_file else b""
-        command_str = (
+
+        if self.flags:
+            command_str = b"\n".join([f.encode("utf-8") for f in self.flags])
+            command_str += b"\n"
+        else:
+            command_str = b""
+
+        command_str += (
             b"\n".join([c.encode("utf-8") for c in commands])
             + b"\n"
             + filename
@@ -259,16 +273,21 @@ class ExifTool:
 
         # read the output
         output = b""
+        warning = b""
         error = b""
         while EXIFTOOL_STAYOPEN_EOF not in str(output):
             line = self._process.stdout.readline()
             if line.startswith(b"Warning"):
-                error += line
+                warning += line.strip()
+            elif line.startswith(b"Error"):
+                error += line.strip()
             else:
                 output += line.strip()
-        error = None if error == b"" else error
+        warning = "" if warning == b"" else warning.decode("utf-8")
+        error = "" if error == b"" else error.decode("utf-8")
+        self.warning = warning
         self.error = error
-        return output[:-EXIFTOOL_STAYOPEN_EOF_LEN], error
+        return output[:-EXIFTOOL_STAYOPEN_EOF_LEN], warning, error
 
     @property
     def pid(self):
@@ -278,23 +297,34 @@ class ExifTool:
     @property
     def version(self):
         """ returns exiftool version """
-        ver, _ = self.run_commands("-ver", no_file=True)
+        ver, _, _ = self.run_commands("-ver", no_file=True)
         return ver.decode("utf-8")
 
-    def asdict(self):
-        """ return dictionary of all EXIF tags and values from exiftool 
-            returns empty dict if no tags
+    def asdict(self, tag_groups=True):
+        """return dictionary of all EXIF tags and values from exiftool
+        returns empty dict if no tags
+
+        Args:
+            tag_groups: if True (default), dict keys have tag groups, e.g. "IPTC:Keywords"; if False, drops groups from keys, e.g. "Keywords"
         """
-        json_str, _ = self.run_commands("-json")
-        if json_str:
-            exifdict = json.loads(json_str)
-            return exifdict[0]
-        else:
+        json_str, _, _ = self.run_commands("-json")
+        if not json_str:
             return dict()
 
+        exifdict = json.loads(json_str)
+        exifdict = exifdict[0]
+        if not tag_groups:
+            # strip tag groups
+            exif_new = {}
+            for k, v in exifdict.items():
+                k = re.sub(r".*:", "", k)
+                exif_new[k] = v
+            exifdict = exif_new
+        return exifdict
+
     def json(self):
         """ returns JSON string containing all EXIF tags and values from exiftool """
-        json, _ = self.run_commands("-json")
+        json, _, _ = self.run_commands("-json")
         return json
 
     def _read_exif(self):
@@ -314,4 +344,5 @@ class ExifTool:
         if exc_type:
             return False
         elif self._commands:
-            _, self.error = self.run_commands(*self._commands)
+            # run_commands sets self.warning and self.error as needed
+            self.run_commands(*self._commands)
