@@ -4,6 +4,11 @@ import json
 import logging
 import math
 
+from collections import namedtuple
+
+MWG_RS_Area = namedtuple("MWG_RS_Area", ["x", "y", "h", "w"])
+MPRI_Reg_Rect = namedtuple("MPRI_Reg_Rect", ["x", "y", "h", "w"])
+
 
 class PersonInfo:
     """ Info about a person in the Photos library
@@ -216,6 +221,62 @@ class FaceInfo:
                 logging.warning(f"Could not get photo for uuid: {self.asset_uuid}")
             return self._photo
 
+    @property
+    def mwg_rs_area(self):
+        """ Get coordinates for Metadata Working Group Region Area. 
+
+        Returns:
+            MWG_RS_Area named tuple with x, y, h, w where:
+            x = stArea:x
+            y = stArea:y
+            h = stArea:h
+            w = stArea:w
+
+        Reference:
+            https://photo.stackexchange.com/questions/106410/how-does-xmp-define-the-face-region
+        """
+        x, y = self.center_x, self.center_y
+        x, y = self._fix_orientation((x, y))
+
+        if self.photo.orientation in [5, 6, 7, 8]:
+            w = self.size_pixels / self.photo.height
+            h = self.size_pixels / self.photo.width
+        else:
+            h = self.size_pixels / self.photo.height
+            w = self.size_pixels / self.photo.width
+
+        return MWG_RS_Area(x, y, h, w)
+
+    @property
+    def mpri_reg_rect(self):
+        """ Get coordinates for Microsoft Photo Region Rectangle.
+
+        Returns:
+            MPRI_Reg_Rect named tuple with x, y, h, w where:
+            x = x coordinate of top left corner of rectangle
+            y = y coordinate of top left corner of rectangle
+            h = height of rectangle
+            w = width of rectangle
+
+        Reference:
+            https://docs.microsoft.com/en-us/windows/win32/wic/-wic-people-tagging
+        """
+        x, y = self.center_x, self.center_y
+        x, y = self._fix_orientation((x, y))
+
+        if self.photo.orientation in [5, 6, 7, 8]:
+            w = self.size_pixels / self.photo.width
+            h = self.size_pixels / self.photo.height
+            x = x - self.size_pixels / self.photo.height / 2
+            y = y - self.size_pixels / self.photo.width / 2
+        else:
+            h = self.size_pixels / self.photo.width
+            w = self.size_pixels / self.photo.height
+            x = x - self.size_pixels / self.photo.width / 2
+            y = y - self.size_pixels / self.photo.height / 2
+
+        return MPRI_Reg_Rect(x, y, h, w)
+
     def face_rect(self):
         """ Get face rectangle coordinates for current version of the associated image
             If image has been edited, rectangle applies to edited version, otherwise original version
@@ -259,6 +320,43 @@ class FaceInfo:
         _, _, yaw = self.roll_pitch_yaw()
         return yaw
 
+    def _fix_orientation(self, xy):
+        """ Translate an (x, y) tuple based on image orientation
+
+        Arguments:
+            xy: tuple of (x, y) coordinates for point to translate
+                in format used by Photos (percent of height/width)
+        
+        Returns:
+            (x, y) tuple of translated coordinates
+        """
+        # Reference: https://github.com/neilpa/phace/blob/7594776480505d0c389688a42099c94ac5d34f3f/cmd/phace/draw.go#L79-L94
+
+        orientation = self.photo.orientation
+        x, y = xy
+        if orientation == 1:
+            y = 1.0 - y
+        elif orientation == 2:
+            y = 1.0 - y
+            x = 1.0 - x
+        elif orientation == 3:
+            x = 1.0 - x
+        elif orientation == 4:
+            pass
+        elif orientation == 5:
+            x, y = 1.0 - y, x
+        elif orientation == 6:
+            x, y = 1.0 - y, 1.0 - x
+        elif orientation == 7:
+            x, y = y, x
+            y = 1.0 - y
+        elif orientation ==8:
+            x, y = y, x
+        else:
+            logging.warning(f"Unhandled orientation: {orientation}")
+
+        return (x, y)
+
     def _make_point(self, xy):
         """ Translate an (x, y) tuple based on image orientation
             and convert to image coordinates
@@ -273,22 +371,11 @@ class FaceInfo:
         # Reference: https://github.com/neilpa/phace/blob/7594776480505d0c389688a42099c94ac5d34f3f/cmd/phace/draw.go#L79-L94
 
         orientation = self.photo.orientation
-        x, y = xy
+        x, y = self._fix_orientation(xy)
         dx = self.photo.width
         dy = self.photo.height
-        if orientation in [1, 2]:
-            y = 1.0 - y
-        elif orientation in [3, 4]:
-            x = 1.0 - x
-        elif orientation in [5, 6]:
-            x, y = 1.0 - y, 1.0 - x
+        if orientation in [5, 6, 7, 8]:
             dx, dy = dy, dx
-        elif orientation in [7, 8]:
-            x, y = y, x
-            dx, dy = dy, dx
-        else:
-            logging.warning(f"Unhandled orientation: {orientation}")
-
         return (int(x * dx), int(y * dy))
 
     def _make_point_with_rotation(self, xy):
@@ -336,6 +423,8 @@ class FaceInfo:
             "right_eye": self.right_eye,
             "size": self.size,
             "face_rect": self.face_rect(),
+            "mpri_reg_rect": self.mpri_reg_rect._asdict(),
+            "mwg_rs_area": self.mwg_rs_area._asdict(),
             "roll": roll,
             "pitch": pitch,
             "yaw": yaw,
