@@ -21,11 +21,11 @@ from .._constants import (
     _PHOTOS_4_ALBUM_KIND,
     _PHOTOS_4_ROOT_FOLDER,
     _PHOTOS_4_VERSION,
-    _PHOTOS_5_VERSION,
     _PHOTOS_5_ALBUM_KIND,
     _PHOTOS_5_IMPORT_SESSION_ALBUM_KIND,
     _PHOTOS_5_SHARED_ALBUM_KIND,
     _PHOTOS_5_SHARED_PHOTO_PATH,
+    _PHOTOS_5_VERSION,
 )
 from ..albuminfo import AlbumInfo, ImportInfo
 from ..personinfo import FaceInfo, PersonInfo
@@ -43,6 +43,7 @@ class PhotoInfo:
     # import additional methods
     from ._photoinfo_searchinfo import (
         search_info,
+        search_info_normalized,
         labels,
         labels_normalized,
         SearchInfo,
@@ -55,6 +56,8 @@ class PhotoInfo:
         _export_photo,
         _exiftool_dict,
         _exiftool_json_sidecar,
+        _get_exif_keywords,
+        _get_exif_persons,
         _write_exif_data,
         _write_sidecar,
         _xmp_sidecar,
@@ -83,8 +86,8 @@ class PhotoInfo:
 
     @property
     def original_filename(self):
-        """ original filename of the picture 
-            Photos 5 mangles filenames upon import """
+        """original filename of the picture
+        Photos 5 mangles filenames upon import"""
         if (
             self._db._db_version <= _PHOTOS_4_VERSION
             and self.has_raw
@@ -103,22 +106,22 @@ class PhotoInfo:
 
     @property
     def date_modified(self):
-        """ image modification date as timezone aware datetime object
-            or None if no modification date set """
+        """image modification date as timezone aware datetime object
+        or None if no modification date set"""
 
         # Photos <= 4 provides no way to get date of adjustment and will update
         # lastmodifieddate anytime photo database record is updated (e.g. adding tags)
         # only report lastmodified date for Photos <=4 if photo is edited;
         # even in this case, the date could be incorrect
-        if self.hasadjustments or self._db._db_version > _PHOTOS_4_VERSION:
-            imagedate = self._info["lastmodifieddate"]
-            if imagedate:
-                seconds = self._info["imageTimeZoneOffsetSeconds"] or 0
-                delta = timedelta(seconds=seconds)
-                tz = timezone(delta)
-                return imagedate.astimezone(tz=tz)
-            else:
-                return None
+        if not self.hasadjustments and self._db._db_version <= _PHOTOS_4_VERSION:
+            return None
+
+        imagedate = self._info["lastmodifieddate"]
+        if imagedate:
+            seconds = self._info["imageTimeZoneOffsetSeconds"] or 0
+            delta = timedelta(seconds=seconds)
+            tz = timezone(delta)
+            return imagedate.astimezone(tz=tz)
         else:
             return None
 
@@ -135,6 +138,7 @@ class PhotoInfo:
         except AttributeError:
             self._path = None
             photopath = None
+            # TODO: should path try to return path even if ismissing?
             if self._info["isMissing"] == 1:
                 return photopath  # path would be meaningless until downloaded
 
@@ -489,46 +493,61 @@ class PhotoInfo:
 
     @property
     def ismissing(self):
-        """ returns true if photo is missing from disk (which means it's not been downloaded from iCloud) 
+        """returns true if photo is missing from disk (which means it's not been downloaded from iCloud)
         NOTE:   the photos.db database uses an asynchrounous write-ahead log so changes in Photos
-                do not immediately get written to disk. In particular, I've noticed that downloading 
+                do not immediately get written to disk. In particular, I've noticed that downloading
                 an image from the cloud does not force the database to be updated until something else
                 e.g. an edit, keyword, etc. occurs forcing a database synch
                 The exact process / timing is a mystery to be but be aware that if some photos were recently
                 downloaded from cloud to local storate their status in the database might still show
                 isMissing = 1
         """
-        return True if self._info["isMissing"] == 1 else False
+        return self._info["isMissing"] == 1
 
     @property
     def hasadjustments(self):
         """ True if picture has adjustments / edits """
-        return True if self._info["hasAdjustments"] == 1 else False
+        return self._info["hasAdjustments"] == 1
 
     @property
     def external_edit(self):
         """ Returns True if picture was edited outside of Photos using external editor """
-        return (
-            True
-            if self._info["adjustmentFormatID"] == "com.apple.Photos.externalEdit"
-            else False
-        )
+        return self._info["adjustmentFormatID"] == "com.apple.Photos.externalEdit"
 
     @property
     def favorite(self):
         """ True if picture is marked as favorite """
-        return True if self._info["favorite"] == 1 else False
+        return self._info["favorite"] == 1
 
     @property
     def hidden(self):
         """ True if picture is hidden """
-        return True if self._info["hidden"] == 1 else False
+        return self._info["hidden"] == 1
+
+    @property
+    def visible(self):
+        """ True if picture is visble """
+        return self._info["visible"]
 
     @property
     def intrash(self):
         """ True if picture is in trash ('Recently Deleted' folder)"""
         return self._info["intrash"]
 
+    @property
+    def date_trashed(self):
+        """ Date asset was placed in the trash or None """
+        # TODO: add add_timezone(dt, offset_seconds) to datetime_utils
+        # also update date_modified
+        trasheddate = self._info["trasheddate"]
+        if trasheddate:
+            seconds = self._info["imageTimeZoneOffsetSeconds"] or 0
+            delta = timedelta(seconds=seconds)
+            tz = timezone(delta)
+            return trasheddate.astimezone(tz=tz)
+        else:
+            return None
+
     @property
     def location(self):
         """ returns (latitude, longitude) as float in degrees or None """
@@ -536,8 +555,8 @@ class PhotoInfo:
 
     @property
     def shared(self):
-        """ returns True if photos is in a shared iCloud album otherwise false
-            Only valid on Photos 5; returns None on older versions """
+        """returns True if photos is in a shared iCloud album otherwise false
+        Only valid on Photos 5; returns None on older versions"""
         if self._db._db_version > _PHOTOS_4_VERSION:
             return self._info["shared"]
         else:
@@ -545,24 +564,25 @@ class PhotoInfo:
 
     @property
     def uti(self):
-        """ Returns Uniform Type Identifier (UTI) for the image
-            for example: public.jpeg or com.apple.quicktime-movie
+        """Returns Uniform Type Identifier (UTI) for the image
+        for example: public.jpeg or com.apple.quicktime-movie
         """
-        if self._db._db_version <= _PHOTOS_4_VERSION:
-            if self.hasadjustments:
-                return self._info["UTI_edited"]
-            elif self.has_raw and self.raw_original:
-                # return UTI of the non-raw image to match Photos 5+ behavior
-                return self._info["raw_pair_info"]["UTI"]
-            else:
-                return self._info["UTI"]
+        if self._db._db_version <= _PHOTOS_4_VERSION and self.hasadjustments:
+            return self._info["UTI_edited"]
+        elif (
+            self._db._db_version <= _PHOTOS_4_VERSION
+            and self.has_raw
+            and self.raw_original
+        ):
+            # return UTI of the non-raw image to match Photos 5+ behavior
+            return self._info["raw_pair_info"]["UTI"]
         else:
             return self._info["UTI"]
 
     @property
     def uti_original(self):
-        """ Returns Uniform Type Identifier (UTI) for the original image
-            for example: public.jpeg or com.apple.quicktime-movie
+        """Returns Uniform Type Identifier (UTI) for the original image
+        for example: public.jpeg or com.apple.quicktime-movie
         """
         if self._db._db_version <= _PHOTOS_4_VERSION and self._info["has_raw"]:
             return self._info["raw_pair_info"]["UTI"]
@@ -574,9 +594,9 @@ class PhotoInfo:
 
     @property
     def uti_edited(self):
-        """ Returns Uniform Type Identifier (UTI) for the edited image 
-            if the photo has been edited, otherwise None; 
-            for example: public.jpeg 
+        """Returns Uniform Type Identifier (UTI) for the edited image
+        if the photo has been edited, otherwise None;
+        for example: public.jpeg
         """
         if self._db._db_version >= _PHOTOS_5_VERSION:
             return self.uti if self.hasadjustments else None
@@ -585,36 +605,34 @@ class PhotoInfo:
 
     @property
     def uti_raw(self):
-        """ Returns Uniform Type Identifier (UTI) for the RAW image if there is one
-            for example: com.canon.cr2-raw-image
-            Returns None if no associated RAW image
+        """Returns Uniform Type Identifier (UTI) for the RAW image if there is one
+        for example: com.canon.cr2-raw-image
+        Returns None if no associated RAW image
         """
         return self._info["UTI_raw"]
 
     @property
     def ismovie(self):
-        """ Returns True if file is a movie, otherwise False
-        """
-        return True if self._info["type"] == _MOVIE_TYPE else False
+        """Returns True if file is a movie, otherwise False"""
+        return self._info["type"] == _MOVIE_TYPE
 
     @property
     def isphoto(self):
-        """ Returns True if file is an image, otherwise False
-        """
-        return True if self._info["type"] == _PHOTO_TYPE else False
+        """Returns True if file is an image, otherwise False"""
+        return self._info["type"] == _PHOTO_TYPE
 
     @property
     def incloud(self):
-        """ Returns True if photo is cloud asset and is synched to cloud
-                    False if photo is cloud asset and not yet synched to cloud
-                    None if photo is not cloud asset
+        """Returns True if photo is cloud asset and is synched to cloud
+        False if photo is cloud asset and not yet synched to cloud
+        None if photo is not cloud asset
         """
         return self._info["incloud"]
 
     @property
     def iscloudasset(self):
-        """ Returns True if photo is a cloud asset (in an iCloud library),
-            otherwise False 
+        """Returns True if photo is a cloud asset (in an iCloud library),
+        otherwise False
         """
         if self._db._db_version <= _PHOTOS_4_VERSION:
             return (
@@ -626,6 +644,11 @@ class PhotoInfo:
         else:
             return True if self._info["cloudAssetGUID"] is not None else False
 
+    @property
+    def isreference(self):
+        """ Returns True if photo is a reference (not copied to the Photos library), otherwise False """
+        return self._info["isreference"]
+
     @property
     def burst(self):
         """ Returns True if photo is part of a Burst photo set, otherwise False """
@@ -633,9 +656,9 @@ class PhotoInfo:
 
     @property
     def burst_photos(self):
-        """ If photo is a burst photo, returns list of PhotoInfo objects 
-            that are part of the same burst photo set; otherwise returns empty list.
-            self is not included in the returned list """
+        """If photo is a burst photo, returns list of PhotoInfo objects
+        that are part of the same burst photo set; otherwise returns empty list.
+        self is not included in the returned list"""
         if self._info["burst"]:
             burst_uuid = self._info["burstUUID"]
             return [
@@ -653,9 +676,9 @@ class PhotoInfo:
 
     @property
     def path_live_photo(self):
-        """ Returns path to the associated video file for a live photo
-            If photo is not a live photo, returns None
-            If photo is missing, returns None """
+        """Returns path to the associated video file for a live photo
+        If photo is not a live photo, returns None
+        If photo is missing, returns None"""
 
         photopath = None
         if self._db._db_version <= _PHOTOS_4_VERSION:
@@ -782,9 +805,9 @@ class PhotoInfo:
 
     @property
     def raw_original(self):
-        """ returns True if associated raw image and the raw image is selected in Photos
-            via "Use RAW as Original "
-            otherwise returns False """
+        """returns True if associated raw image and the raw image is selected in Photos
+        via "Use RAW as Original "
+        otherwise returns False"""
         return self._info["raw_is_original"]
 
     @property
@@ -831,27 +854,27 @@ class PhotoInfo:
         inplace_sep=None,
         filename=False,
         dirname=False,
-        replacement=":",
+        strip=False,
     ):
         """Renders a template string for PhotoInfo instance using PhotoTemplate
 
         Args:
             template_str: a template string with fields to render
             none_str: a str to use if template field renders to None, default is "_".
-            path_sep: a single character str to use as path separator when joining 
+            path_sep: a single character str to use as path separator when joining
                 fields like folder_album; if not provided, defaults to os.path.sep
-            expand_inplace: expand multi-valued substitutions in-place as a single string 
+            expand_inplace: expand multi-valued substitutions in-place as a single string
                 instead of returning individual strings
             inplace_sep: optional string to use as separator between multi-valued keywords
                 with expand_inplace; default is ','
             filename: if True, template output will be sanitized to produce valid file name
-            dirname: if True, template output will be sanitized to produce valid directory name 
-            replacement: str, value to replace any illegal file path characters with; default = ":"
-        
+            dirname: if True, template output will be sanitized to produce valid directory name
+            strip: if True, strips leading/trailing white space from resulting template
+
         Returns:
             ([rendered_strings], [unmatched]): tuple of list of rendered strings and list of unmatched template values
         """
-        template = PhotoTemplate(self)
+        template = PhotoTemplate(self, exiftool_path=self._db._exiftool_path)
         return template.render(
             template_str,
             none_str=none_str,
@@ -860,7 +883,7 @@ class PhotoInfo:
             inplace_sep=inplace_sep,
             filename=filename,
             dirname=dirname,
-            replacement=replacement,
+            strip=strip,
         )
 
     @property
@@ -874,11 +897,11 @@ class PhotoInfo:
         return self._info["latitude"]
 
     def _get_album_uuids(self):
-        """ Return list of album UUIDs this photo is found in
-        
+        """Return list of album UUIDs this photo is found in
+
             Filters out albums in the trash and any special album types
 
-        Returns: list of album UUIDs 
+        Returns: list of album UUIDs
         """
         if self._db._db_version <= _PHOTOS_4_VERSION:
             version4 = True
@@ -980,6 +1003,7 @@ class PhotoInfo:
         comments = [comment.asdict() for comment in self.comments]
         likes = [like.asdict() for like in self.likes]
         faces = [face.asdict() for face in self.face_info]
+        search_info = self.search_info.asdict() if self.search_info else {}
 
         return {
             "library": self._db._library_path,
@@ -1015,6 +1039,7 @@ class PhotoInfo:
             "path_live_photo": self.path_live_photo,
             "iscloudasset": self.iscloudasset,
             "incloud": self.incloud,
+            "isreference": self.isreference,
             "date_modified": self.date_modified,
             "portrait": self.portrait,
             "screenshot": self.screenshot,
@@ -1041,6 +1066,7 @@ class PhotoInfo:
             "original_filesize": self.original_filesize,
             "comments": comments,
             "likes": likes,
+            "search_info": search_info,
         }
 
     def json(self):
