@@ -6,9 +6,9 @@
 # 2. Needed to handle default values if template not found
 # 3. Didn't want user to need to know python (e.g. by using Mako which is
 #    already used elsewhere in this project)
-# 4. Couldn't figure out how to do #1 and #2 with str.format()
 #
-# This code isn't elegant but it seems to work well.  PRs gladly accepted.
+# This code isn't elegant and is prime for refactoring but it seems to work well.  PRs gladly accepted.
+
 import datetime
 import locale
 import os
@@ -52,6 +52,7 @@ TEMPLATE_SUBSTITUTIONS = {
     ),
     "{photo_or_video}": "'photo' or 'video' depending on what type the image is. To customize, use default value as in '{photo_or_video,photo=fotos;video=videos}'",
     "{hdr}": "Photo is HDR?; True/False value, use in format '{hdr?VALUE_IF_TRUE,VALUE_IF_FALSE}'",
+    "{edited}": "Photo has been edited (has adjustments)?; True/False value, use in format '{edited?VALUE_IF_TRUE,VALUE_IF_FALSE}'",
     "{created.date}": "Photo's creation date in ISO format, e.g. '2020-03-22'",
     "{created.year}": "4-digit year of photo creation time",
     "{created.yy}": "2-digit year of photo creation time",
@@ -69,23 +70,23 @@ TEMPLATE_SUBSTITUTIONS = {
     + "{created.strftime,%Y-%U} would result in year-week number of year: '2020-23'. "
     + "If used with no template will return null value. "
     + "See https://strftime.org/ for help on strftime templates.",
-    "{modified.date}": "Photo's modification date in ISO format, e.g. '2020-03-22'",
-    "{modified.year}": "4-digit year of photo modification time",
-    "{modified.yy}": "2-digit year of photo modification time",
-    "{modified.mm}": "2-digit month of the photo modification time (zero padded)",
-    "{modified.month}": "Month name in user's locale of the photo modification time",
-    "{modified.mon}": "Month abbreviation in the user's locale of the photo modification time",
-    "{modified.dd}": "2-digit day of the month (zero padded) of the photo modification time",
-    "{modified.dow}": "Day of week in user's locale of the photo modification time",
-    "{modified.doy}": "3-digit day of year (e.g Julian day) of photo modification time, starting from 1 (zero padded)",
-    "{modified.hour}": "2-digit hour of the photo modification time",
-    "{modified.min}": "2-digit minute of the photo modification time",
-    "{modified.sec}": "2-digit second of the photo modification time",
-    # "{modified.strftime}": "Apply strftime template to file modification date/time. Should be used in form "
-    # + "{modified.strftime,TEMPLATE} where TEMPLATE is a valid strftime template, e.g. "
-    # + "{modified.strftime,%Y-%U} would result in year-week number of year: '2020-23'. "
-    # + "If used with no template will return null value. "
-    # + "See https://strftime.org/ for help on strftime templates.",
+    "{modified.date}": "Photo's modification date in ISO format, e.g. '2020-03-22'; uses creation date if photo is not modified",
+    "{modified.year}": "4-digit year of photo modification time; uses creation date if photo is not modified",
+    "{modified.yy}": "2-digit year of photo modification time; uses creation date if photo is not modified",
+    "{modified.mm}": "2-digit month of the photo modification time (zero padded); uses creation date if photo is not modified",
+    "{modified.month}": "Month name in user's locale of the photo modification time; uses creation date if photo is not modified",
+    "{modified.mon}": "Month abbreviation in the user's locale of the photo modification time; uses creation date if photo is not modified",
+    "{modified.dd}": "2-digit day of the month (zero padded) of the photo modification time; uses creation date if photo is not modified",
+    "{modified.dow}": "Day of week in user's locale of the photo modification time; uses creation date if photo is not modified",
+    "{modified.doy}": "3-digit day of year (e.g Julian day) of photo modification time, starting from 1 (zero padded); uses creation date if photo is not modified",
+    "{modified.hour}": "2-digit hour of the photo modification time; uses creation date if photo is not modified",
+    "{modified.min}": "2-digit minute of the photo modification time; uses creation date if photo is not modified",
+    "{modified.sec}": "2-digit second of the photo modification time; uses creation date if photo is not modified",
+    "{modified.strftime}": "Apply strftime template to file modification date/time. Should be used in form "
+    + "{modified.strftime,TEMPLATE} where TEMPLATE is a valid strftime template, e.g. "
+    + "{modified.strftime,%Y-%U} would result in year-week number of year: '2020-23'. "
+    + "If used with no template will return null value. Uses creation date if photo is not modified. "
+    + "See https://strftime.org/ for help on strftime templates.",
     "{today.date}": "Current date in iso format, e.g. '2020-03-22'",
     "{today.year}": "4-digit year of current date",
     "{today.yy}": "2-digit year of current date",
@@ -116,6 +117,11 @@ TEMPLATE_SUBSTITUTIONS = {
     "{place.address.postal_code}": "Postal code part of the postal address, e.g. '20009'",
     "{place.address.country}": "Country name of the postal address, e.g. 'United States'",
     "{place.address.country_code}": "ISO country code of the postal address, e.g. 'US'",
+    "{searchinfo.season}": "Season of the year associated with a photo, e.g. 'Summer'; (Photos 5+ only, applied automatically by Photos' image categorization algorithms).",
+    "{exif.camera_make}": "Camera make from original photo's EXIF information as imported by Photos, e.g. 'Apple'",
+    "{exif.camera_model}": "Camera model from original photo's EXIF information as imported by Photos, e.g. 'iPhone 6s'",
+    "{exif.lens_model}": "Lens model from original photo's EXIF information as imported by Photos, e.g. 'iPhone 6s back camera 4.15mm f/2.2'",
+    "{uuid}": "Photo's internal universally unique identifier (UUID) for the photo, a 36-character string unique to the photo, e.g. '128FB4C6-0B16-4E7D-9108-FB2E90DA1546'",
 }
 
 # Permitted multi-value substitutions (each of these returns None or 1 or more values)
@@ -124,13 +130,17 @@ TEMPLATE_SUBSTITUTIONS_MULTI_VALUED = {
     "{folder_album}": "Folder path + album photo is contained in. e.g. 'Folder/Subfolder/Album' or just 'Album' if no enclosing folder",
     "{keyword}": "Keyword(s) assigned to photo",
     "{person}": "Person(s) / face(s) in a photo",
-    "{label}": "Image categorization label associated with a photo (Photos 5 only)",
-    "{label_normalized}": "All lower case version of 'label' (Photos 5 only)",
-    "{comment}": "Comment(s) on shared Photos; format is 'Person name: comment text' (Photos 5 only)",
+    "{label}": "Image categorization label associated with a photo (Photos 5+ only)",
+    "{label_normalized}": "All lower case version of 'label' (Photos 5+ only)",
+    "{comment}": "Comment(s) on shared Photos; format is 'Person name: comment text' (Photos 5+ only)",
     "{exiftool:GROUP:TAGNAME}": "Use exiftool (https://exiftool.org) to extract metadata, in form GROUP:TAGNAME, from image.  "
     "E.g. '{exiftool:EXIF:Make}' to get camera make, or {exiftool:IPTC:Keywords} to extract keywords. "
     "See https://exiftool.org/TagNames/ for list of valid tag names.  You must specify group (e.g. EXIF, IPTC, etc) "
     "as used in `exiftool -G`. exiftool must be installed in the path to use this template.",
+    "{searchinfo.holiday}": "Holiday names associated with a photo, e.g. 'Christmas Day'; (Photos 5+ only, applied automatically by Photos' image categorization algorithms).",
+    "{searchinfo.activity}": "Activities associated with a photo, e.g. 'Sporting Event'; (Photos 5+ only, applied automatically by Photos' image categorization algorithms).",
+    "{searchinfo.venue}": "Venues associated with a photo, e.g. name of restaurant; (Photos 5+ only, applied automatically by Photos' image categorization algorithms).",
+    "{searchinfo.venue_type}": "Venue types associated with a photo, e.g. 'Restaurant'; (Photos 5+ only, applied automatically by Photos' image categorization algorithms).",
 }
 
 # Just the multi-valued substitution names without the braces
@@ -139,25 +149,48 @@ MULTI_VALUE_SUBSTITUTIONS = [
     for field in TEMPLATE_SUBSTITUTIONS_MULTI_VALUED
 ]
 
+# regular expressions for matching template syntax
+RE_OPENING_BRACE = r"(?<!\{)\{"  # match { but not {{
+RE_DELIM = r"([^}]*\+)?"  # group 1: optional DELIM+
+RE_FIELD_NAME = r"([^\\,}+\?]+)"  # group 2: field name
+RE_PATH_SEP = r"(\([^{}\)]*\))?"  # group 3: optional (PATH_SEP)
+# + r"(\[[^{}\)]*\])?"  # group 4: optional [REPLACE]
+RE_REPLACE = r"(\[[^{}]*\])?"  # group 4: optional [REPLACE]
+RE_BOOL_VAL = r"(\?[^\\,}]*)?"  # group 5: optional ?TRUE_VALUE for boolean fields
+RE_DEFAULT_VAL = r"(,[\w\=\;\-\%. ]*)?"  # group 6: optional ,DEFAULT
+RE_CLOSING_BRACE = r"(?=\}(?!\}))\}"  # match } but not }}
+
+MATCH_GROUPS_TOTAL = 6
+MATCH_GROUPS_DELIM = 1
+MATCH_GROUPS_FIELD = 2
+MATCH_GROUPS_PATH_SEP = 3
+MATCH_GROUPS_REPLACE = 4
+MATCH_GROUPS_BOOL_VAL = 5
+MATCH_GROUPS_DEFAULT = 6
+
+# default values for string manipulation template options
+INPLACE_DEFAULT = ","
+PATH_SEP_DEFAULT = os.path.sep
+
 
 class PhotoTemplate:
     """ PhotoTemplate class to render a template string from a PhotoInfo object """
 
-    def __init__(self, photo):
-        """ Inits PhotoTemplate class with photo, non_str, and path_sep
+    def __init__(self, photo, exiftool_path=None):
+        """ Inits PhotoTemplate class with photo
 
         Args:
             photo: a PhotoInfo instance.
+            exiftool_path: optional path to exiftool for use with {exiftool:} template; if not provided, will look for exiftool in $PATH
         """
         self.photo = photo
+        self.exiftool_path = exiftool_path
 
         # holds value of current date/time for {today.x} fields
         # gets initialized in get_template_value
         self.today = None
 
-    def make_subst_function(
-        self, none_str, filename, dirname, replacement, get_func=None
-    ):
+    def make_subst_function(self, none_str, filename, dirname, get_func=None):
         """ returns: substitution function for use in re.sub 
             none_str: value to use if substitution lookup is None and no default provided
             get_func: function that gets the substitution value for a given template field
@@ -166,37 +199,39 @@ class PhotoTemplate:
         if get_func is None:
             # used by make_subst_function to get the value for a template substitution
             get_func = partial(
-                self.get_template_value,
-                filename=filename,
-                dirname=dirname,
-                replacement=replacement,
+                self.get_template_value, filename=filename, dirname=dirname
             )
 
         # closure to capture photo, none_str, filename, dirname in subst
         def subst(matchobj):
             groups = len(matchobj.groups())
-            if groups != 5:
+            if groups != MATCH_GROUPS_TOTAL:
                 raise ValueError(
-                    f"Unexpected number of groups: expected 4, got {groups}"
+                    f"Unexpected number of groups: expected {MATCH_GROUPS_TOTAL}, got {groups}"
                 )
 
-            delim = matchobj.group(1)
-            field = matchobj.group(2)
-            path_sep = matchobj.group(3)
-            bool_val = matchobj.group(4)
-            default = matchobj.group(5)
+            delim = matchobj.group(MATCH_GROUPS_DELIM)
+            field = matchobj.group(MATCH_GROUPS_FIELD)
+            path_sep = matchobj.group(MATCH_GROUPS_PATH_SEP)
+            replace = matchobj.group(MATCH_GROUPS_REPLACE)
+            bool_val = matchobj.group(MATCH_GROUPS_BOOL_VAL)
+            default = matchobj.group(MATCH_GROUPS_DEFAULT)
 
             # drop the '+' on delim
             delim = delim[:-1] if delim is not None else None
             # drop () from path_sep
             path_sep = path_sep.strip("()") if path_sep is not None else None
+            # drop [] from replace
+            replace = replace[1:-1] if replace is not None else None
             # drop the ? on bool_val
             bool_val = bool_val[1:] if bool_val is not None else None
             # drop the comma on default
             default_val = default[1:] if default is not None else None
 
             try:
-                val = get_func(field, default_val, bool_val, delim, path_sep)
+                val = get_func(
+                    field, default_val, bool_val, delim, path_sep, replacement=replace
+                )
             except ValueError:
                 return matchobj.group(0)
 
@@ -220,7 +255,7 @@ class PhotoTemplate:
         inplace_sep=None,
         filename=False,
         dirname=False,
-        replacement=":",
+        strip=False,
     ):
         """ Render a filename or directory template 
 
@@ -234,17 +269,17 @@ class PhotoTemplate:
             with expand_inplace; default is ','
             filename: if True, template output will be sanitized to produce valid file name
             dirname: if True, template output will be sanitized to produce valid directory name 
-            replacement: str, value to replace any illegal file path characters with; default = ":"
+            strip: if True, strips leading/trailing whitespace from rendered templates
 
         Returns:
             ([rendered_strings], [unmatched]): tuple of list of rendered strings and list of unmatched template values
         """
 
         if path_sep is None:
-            path_sep = os.path.sep
+            path_sep = PATH_SEP_DEFAULT
 
         if inplace_sep is None:
-            inplace_sep = ","
+            inplace_sep = INPLACE_DEFAULT
 
         # the rendering happens in two phases:
         # phase 1: handle all the single-value template substitutions
@@ -257,19 +292,20 @@ class PhotoTemplate:
         # regex to find {template_field,optional_default} in strings
         # pylint: disable=anomalous-backslash-in-string
         regex = (
-            r"(?<!\{)\{"  # match { but not {{
-            + r"([^}]*\+)?"  # group 1: optional DELIM+
-            + r"([^\\,}+\?]+)"  # group 2: field name
-            + r"(\([^{}\)]*\))?"  # group 3: optional (PATH_SEP)
-            + r"(\?[^\\,}]*)?"  # group 4: optional ?TRUE_VALUE for boolean fields
-            + r"(,[\w\=\;\-\%. ]*)?"  # group 5: optional ,DEFAULT
-            + r"(?=\}(?!\}))\}"  # match } but not }}
+            RE_OPENING_BRACE
+            + RE_DELIM
+            + RE_FIELD_NAME
+            + RE_PATH_SEP
+            + RE_REPLACE
+            + RE_BOOL_VAL
+            + RE_DEFAULT_VAL
+            + RE_CLOSING_BRACE
         )
 
         if type(template) is not str:
             raise TypeError(f"template must be type str, not {type(template)}")
 
-        subst_func = self.make_subst_function(none_str, filename, dirname, replacement)
+        subst_func = self.make_subst_function(none_str, filename, dirname)
 
         # do the replacements
         rendered = re.sub(regex, subst_func, template)
@@ -298,14 +334,7 @@ class PhotoTemplate:
         #    '2011/Album2/keyword2/person1',]
 
         rendered_strings = self._render_multi_valued_templates(
-            rendered,
-            none_str,
-            path_sep,
-            expand_inplace,
-            inplace_sep,
-            filename,
-            dirname,
-            replacement,
+            rendered, none_str, path_sep, expand_inplace, inplace_sep, filename, dirname
         )
 
         # process exiftool: templates
@@ -317,7 +346,6 @@ class PhotoTemplate:
             inplace_sep,
             filename,
             dirname,
-            replacement,
         )
 
         # find any {fields} that weren't replaced
@@ -342,6 +370,11 @@ class PhotoTemplate:
                 sanitize_filename(rendered_str) for rendered_str in rendered_strings
             ]
 
+        if strip:
+            rendered_strings = [
+                rendered_str.strip() for rendered_str in rendered_strings
+            ]
+
         return rendered_strings, unmatched
 
     def _render_multi_valued_templates(
@@ -353,7 +386,6 @@ class PhotoTemplate:
         inplace_sep,
         filename,
         dirname,
-        replacement,
     ):
         rendered_strings = [rendered]
         new_rendered_strings = []
@@ -362,15 +394,16 @@ class PhotoTemplate:
             for field in MULTI_VALUE_SUBSTITUTIONS:
                 # Build a regex that matches only the field being processed
                 re_str = (
-                    r"(?<!\{)\{"  # match { but not {{
-                    + r"([^}]*\+)?"  # group 1: optional DELIM+
+                    RE_OPENING_BRACE
+                    + RE_DELIM
                     + r"("
                     + field  # group 2: field name
                     + r")"
-                    + r"(\([^{}\)]*\))?"  # group 3: optional (PATH_SEP)
-                    + r"(\?[^\\,}]*)?"  # group 4: optional ?TRUE_VALUE for boolean fields
-                    + r"(,[\w\=\;\-\%. ]*)?"  # group 5: optional ,DEFAULT
-                    + r"(?=\}(?!\}))\}"  # match } but not }}
+                    + RE_PATH_SEP
+                    + RE_REPLACE
+                    + RE_BOOL_VAL
+                    + RE_DEFAULT_VAL
+                    + RE_CLOSING_BRACE
                 )
                 regex_multi = re.compile(re_str)
 
@@ -381,21 +414,29 @@ class PhotoTemplate:
                     matches = regex_multi.search(str_template)
                     if matches:
                         path_sep = (
-                            matches.group(3).strip("()")
-                            if matches.group(3) is not None
+                            matches.group(MATCH_GROUPS_PATH_SEP).strip("()")
+                            if matches.group(MATCH_GROUPS_PATH_SEP) is not None
                             else path_sep
                         )
+                        replace = (
+                            matches.group(MATCH_GROUPS_REPLACE)[1:-1]
+                            if matches.group(MATCH_GROUPS_REPLACE) is not None
+                            else None
+                        )
                         values = self.get_template_value_multi(
                             field,
                             path_sep,
                             filename=filename,
                             dirname=dirname,
-                            replacement=replacement,
+                            replacement=replace,
                         )
-                        if expand_inplace or matches.group(1) is not None:
+                        if (
+                            expand_inplace
+                            or matches.group(MATCH_GROUPS_DELIM) is not None
+                        ):
                             delim = (
-                                matches.group(1)[:-1]
-                                if matches.group(1) is not None
+                                matches.group(MATCH_GROUPS_DELIM)[:-1]
+                                if matches.group(MATCH_GROUPS_DELIM) is not None
                                 else inplace_sep
                             )
                             # instead of returning multiple strings, join values into a single string
@@ -405,7 +446,9 @@ class PhotoTemplate:
                                 else None
                             )
 
-                            def lookup_template_value_multi(lookup_value, *_):
+                            def lookup_template_value_multi(
+                                lookup_value, *args, **kwargs
+                            ):
                                 """ Closure passed to make_subst_function get_func 
                                         Capture val and field in the closure 
                                         Allows make_subst_function to be re-used w/o modification
@@ -421,7 +464,6 @@ class PhotoTemplate:
                                 none_str,
                                 filename,
                                 dirname,
-                                replacement,
                                 get_func=lookup_template_value_multi,
                             )
                             new_string = regex_multi.sub(subst, str_template)
@@ -432,7 +474,9 @@ class PhotoTemplate:
                             # create a new template string for each value
                             for val in values:
 
-                                def lookup_template_value_multi(lookup_value, *_):
+                                def lookup_template_value_multi(
+                                    lookup_value, *args, **kwargs
+                                ):
                                     """ Closure passed to make_subst_function get_func 
                                         Capture val and field in the closure 
                                         Allows make_subst_function to be re-used w/o modification
@@ -448,7 +492,6 @@ class PhotoTemplate:
                                     none_str,
                                     filename,
                                     dirname,
-                                    replacement,
                                     get_func=lookup_template_value_multi,
                                 )
                                 new_string = regex_multi.sub(subst, str_template)
@@ -467,7 +510,6 @@ class PhotoTemplate:
         inplace_sep,
         filename,
         dirname,
-        replacement,
     ):
         # TODO: lots of code commonality with render_multi_valued_templates -- combine or pull out
         # TODO: put these in globals
@@ -478,15 +520,15 @@ class PhotoTemplate:
             inplace_sep = ","
 
         # Build a regex that matches only the field being processed
-        # todo: pull out regexes into globals?
         re_str = (
-            r"(?<!\{)\{"  # match { but not {{
-            + r"([^}]*\+)?"  # group 1: optional DELIM+
-            + r"(exiftool:[^\\,}+\?]+)"  # group 3 field name
-            + r"(\([^{}\)]*\))?"  # group 3: optional (PATH_SEP)
-            + r"(\?[^\\,}]*)?"  # group 4: optional ?TRUE_VALUE for boolean fields
-            + r"(,[\w\=\;\-\%. ]*)?"  # group 5: optional ,DEFAULT
-            + r"(?=\}(?!\}))\}"  # match } but not }}
+            RE_OPENING_BRACE
+            + RE_DELIM
+            + r"(exiftool:[^\\,}+\?\[\]]+)"  # group 3 field name
+            + RE_PATH_SEP
+            + RE_REPLACE
+            + RE_BOOL_VAL
+            + RE_DEFAULT_VAL
+            + RE_CLOSING_BRACE
         )
         regex_multi = re.compile(re_str)
 
@@ -501,17 +543,21 @@ class PhotoTemplate:
                     # allmatches = regex_multi.finditer(str_template)
                     # for matches in allmatches:
                     path_sep = (
-                        matches.group(3).strip("()")
-                        if matches.group(3) is not None
+                        matches.group(MATCH_GROUPS_PATH_SEP).strip("()")
+                        if matches.group(MATCH_GROUPS_PATH_SEP) is not None
                         else path_sep
                     )
-                    field = matches.group(2)
+                    replace = (
+                        matches.group(MATCH_GROUPS_REPLACE)[1:-1]
+                        if matches.group(MATCH_GROUPS_REPLACE) is not None
+                        else None
+                    )
+                    field = matches.group(MATCH_GROUPS_FIELD)
                     subfield = field[9:]
-
                     if not self.photo.path:
-                        values = []
+                        values = [None]
                     else:
-                        exif = ExifTool(self.photo.path)
+                        exif = ExifTool(self.photo.path, exiftool=self.exiftool_path)
                         exifdict = exif.asdict()
                         exifdict = {k.lower(): v for (k, v) in exifdict.items()}
                         subfield = subfield.lower()
@@ -520,12 +566,24 @@ class PhotoTemplate:
                             values = (
                                 [values] if not isinstance(values, list) else values
                             )
+                            if replace and values:
+                                new_values = []
+                                for value in values:
+                                    new_values.append(self.replace(value, replace))
+                                values = new_values
+
+                            # sanitize directory names if needed
+                            if filename:
+                                values = [sanitize_pathpart(value) for value in values]
+                            elif dirname:
+                                values = [sanitize_dirname(value) for value in values]
+
                         else:
                             values = [None]
-                    if expand_inplace or matches.group(1) is not None:
+                    if expand_inplace or matches.group(MATCH_GROUPS_DELIM) is not None:
                         delim = (
-                            matches.group(1)[:-1]
-                            if matches.group(1) is not None
+                            matches.group(MATCH_GROUPS_DELIM)[:-1]
+                            if matches.group(MATCH_GROUPS_DELIM) is not None
                             else inplace_sep
                         )
                         # instead of returning multiple strings, join values into a single string
@@ -533,7 +591,7 @@ class PhotoTemplate:
                             delim.join(sorted(values)) if values and values[0] else None
                         )
 
-                        def lookup_template_value_exif(lookup_value, *_):
+                        def lookup_template_value_exif(lookup_value, *args, **kwargs):
                             """ Closure passed to make_subst_function get_func 
                                     Capture val and field in the closure 
                                     Allows make_subst_function to be re-used w/o modification
@@ -547,7 +605,6 @@ class PhotoTemplate:
                             none_str,
                             filename,
                             dirname,
-                            replacement,
                             get_func=lookup_template_value_exif,
                         )
                         new_string = regex_multi.sub(subst, str_template)
@@ -557,7 +614,9 @@ class PhotoTemplate:
                         # create a new template string for each value
                         for val in values:
 
-                            def lookup_template_value_exif(lookup_value, *_):
+                            def lookup_template_value_exif(
+                                lookup_value, *args, **kwargs
+                            ):
                                 """ Closure passed to make_subst_function get_func 
                                     Capture val and field in the closure 
                                     Allows make_subst_function to be re-used w/o modification
@@ -573,7 +632,6 @@ class PhotoTemplate:
                                 none_str,
                                 filename,
                                 dirname,
-                                replacement,
                                 get_func=lookup_template_value_exif,
                             )
                             new_string = regex_multi.sub(subst, str_template)
@@ -591,7 +649,7 @@ class PhotoTemplate:
         path_sep=None,
         filename=False,
         dirname=False,
-        replacement=":",
+        replacement=None,
     ):
         """lookup value for template field (single-value template substitutions)
 
@@ -632,7 +690,9 @@ class PhotoTemplate:
         elif field == "photo_or_video":
             value = self.get_photo_video_type(default)
         elif field == "hdr":
-            value = self.get_photo_hdr(default, bool_val)
+            value = self.get_photo_bool_attribute("hdr", default, bool_val)
+        elif field == "edited":
+            value = self.get_photo_bool_attribute("hasadjustments", default, bool_val)
         elif field == "created.date":
             value = DateTimeFormatter(self.photo.date).date
         elif field == "created.year":
@@ -669,74 +729,83 @@ class PhotoTemplate:
             value = (
                 DateTimeFormatter(self.photo.date_modified).date
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).date
             )
         elif field == "modified.year":
             value = (
                 DateTimeFormatter(self.photo.date_modified).year
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).year
             )
         elif field == "modified.yy":
             value = (
                 DateTimeFormatter(self.photo.date_modified).yy
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).yy
             )
         elif field == "modified.mm":
             value = (
                 DateTimeFormatter(self.photo.date_modified).mm
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).mm
             )
         elif field == "modified.month":
             value = (
                 DateTimeFormatter(self.photo.date_modified).month
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).month
             )
         elif field == "modified.mon":
             value = (
                 DateTimeFormatter(self.photo.date_modified).mon
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).mon
             )
         elif field == "modified.dd":
             value = (
                 DateTimeFormatter(self.photo.date_modified).dd
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).dd
             )
         elif field == "modified.dow":
             value = (
                 DateTimeFormatter(self.photo.date_modified).dow
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).dow
             )
         elif field == "modified.doy":
             value = (
                 DateTimeFormatter(self.photo.date_modified).doy
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).doy
             )
         elif field == "modified.hour":
             value = (
                 DateTimeFormatter(self.photo.date_modified).hour
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).hour
             )
         elif field == "modified.min":
             value = (
                 DateTimeFormatter(self.photo.date_modified).min
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).min
             )
         elif field == "modified.sec":
             value = (
                 DateTimeFormatter(self.photo.date_modified).sec
                 if self.photo.date_modified
-                else None
+                else DateTimeFormatter(self.photo.date).sec
             )
+        elif field == "modified.strftime":
+            if default:
+                try:
+                    date = self.photo.date_modified or self.photo.date
+                    value = date.strftime(default)
+                except:
+                    raise ValueError(f"Invalid strftime template: '{default}'")
+            else:
+                value = None
         elif field == "today.date":
             value = DateTimeFormatter(self.today).date
         elif field == "today.year":
@@ -839,18 +908,58 @@ class PhotoTemplate:
                 if self.photo.place and self.photo.place.address.iso_country_code
                 else None
             )
+        elif field == "searchinfo.season":
+            value = self.photo.search_info.season if self.photo.search_info else None
+        elif field == "exif.camera_make":
+            value = self.photo.exif_info.camera_make if self.photo.exif_info else None
+        elif field == "exif.camera_model":
+            value = self.photo.exif_info.camera_model if self.photo.exif_info else None
+        elif field == "exif.lens_model":
+            value = self.photo.exif_info.lens_model if self.photo.exif_info else None
+        elif field == "uuid":
+            value = self.photo.uuid
         else:
             # if here, didn't get a match
             raise ValueError(f"Unhandled template value: {field}")
 
+        if value and replacement:
+            value = self.replace(value, replacement)
+            # process character replacements
+
         if filename:
-            value = sanitize_pathpart(value, replacement=replacement)
+            value = sanitize_pathpart(value)
         elif dirname:
-            value = sanitize_dirname(value, replacement=replacement)
+            value = sanitize_dirname(value)
+
+        return value
+
+    def replace(self, value, replacement):
+        """ process REPLACE template option
+
+        Args:
+            value: str value to process
+            replacement: str in form OLD,NEW|OLD,NEW... with old and new values for replacement
+        
+        Returns:
+            value with all replacements done
+        
+        Raises:
+            ValueError if replacement string is in wrong format
+        """
+        if not value:
+            return value
+
+        replacements = replacement.split("|")
+        for r in replacements:
+            try:
+                old, new = r.split(",")
+            except ValueError:
+                raise ValueError(f"Invalid template REPLACE value: {replacement}")
+            value = value.replace(old, new)
         return value
 
     def get_template_value_multi(
-        self, field, path_sep, filename=False, dirname=False, replacement=":"
+        self, field, path_sep, filename=False, dirname=False, replacement=None
     ):
         """lookup value for template field (multi-value template substitutions)
 
@@ -867,7 +976,7 @@ class PhotoTemplate:
         """
 
         """ return list of values for a multi-valued template field """
-        values = [] 
+        values = []
         if field == "album":
             values = self.photo.albums
         elif field == "keyword":
@@ -889,12 +998,9 @@ class PhotoTemplate:
                     if dirname:
                         # being used as a filepath so sanitize each part
                         folder = path_sep.join(
-                            sanitize_dirname(f, replacement=replacement)
-                            for f in album.folder_names
-                        )
-                        folder += path_sep + sanitize_dirname(
-                            album.title, replacement=replacement
+                            sanitize_dirname(f) for f in album.folder_names
                         )
+                        folder += path_sep + sanitize_dirname(album.title)
                     else:
                         folder = path_sep.join(album.folder_names)
                         folder += path_sep + album.title
@@ -902,28 +1008,41 @@ class PhotoTemplate:
                 else:
                     # album not in folder
                     if dirname:
-                        values.append(
-                            sanitize_dirname(album.title, replacement=replacement)
-                        )
+                        values.append(sanitize_dirname(album.title))
                     else:
                         values.append(album.title)
         elif field == "comment":
             values = [
                 f"{comment.user}: {comment.text}" for comment in self.photo.comments
             ]
+        elif field == "searchinfo.holiday":
+            values = self.photo.search_info.holidays if self.photo.search_info else []
+        elif field == "searchinfo.activity":
+            values = self.photo.search_info.activities if self.photo.search_info else []
+        elif field == "searchinfo.venue":
+            values = self.photo.search_info.venues if self.photo.search_info else []
+        elif field == "searchinfo.venue_type":
+            values = (
+                self.photo.search_info.venue_types if self.photo.search_info else []
+            )
         elif not field.startswith("exiftool:"):
+            # exiftool: templates handled by _render_exiftool_template
             raise ValueError(f"Unhandled template value: {field}")
 
+        # do any replacements needs
+        if replacement:
+            new_values = []
+            for value in values:
+                # process replacements
+                new_values.append(self.replace(value, replacement))
+            values = new_values
+
         # sanitize directory names if needed, folder_album handled differently above
         if filename:
-            values = [
-                sanitize_pathpart(value, replacement=replacement) for value in values
-            ]
+            values = [sanitize_pathpart(value) for value in values]
         elif dirname and field != "folder_album":
             # skip folder_album because it would have been handled above
-            values = [
-                sanitize_dirname(value, replacement=replacement) for value in values
-            ]
+            values = [sanitize_dirname(value) for value in values]
 
         # If no values, insert None so code below will substite none_str for None
         values = values or [None]
@@ -962,8 +1081,10 @@ class PhotoTemplate:
         else:
             return default_dict["photo"]
 
-    def get_photo_hdr(self, default, bool_val):
-        if self.photo.hdr:
+    def get_photo_bool_attribute(self, attr, default, bool_val):
+        # get value for a PhotoInfo bool attribute
+        val = getattr(self.photo, attr)
+        if val:
             return bool_val
         else:
             return default
